% NHST Simulation: Generating distributions of p-values and observing (1 -
% p/pp)

% Author: Patrick R. Heck, 06/2016.
% Written in MATLAB 7.10.0 (R2010a)

% Settings (can change in code): 
% Iter = 10000 %number of means to compute
% p(H) = 0, .5, or .8 %proportion of samples to draw from Ho and Ha
% N = 10 %number of samples to draw for each iteration

%Note: all tests against the alternative hypothesis Ha are commented out
%but can be run.

%current settings take ~ 5 minutes to run. Decrease Iter for faster speed.

clear all

%Create population distributions

Ho = 50 + 10.*randn(100000,1); %mean = 50, SD = 10
Ha = (mean(Ho) + .2*std(Ho)) + 10.*randn(100000,1); %+.5 SD above Ho (M = 55, SD = 10)

Iter = 5000; %number of iterations (means to compute)
N = 20; %number of samples to compute each mean
pHo = .8; %proportion of samples to come from Ho
pHa = (1 - pHo); %proportion of samples to come from Ha

distSamples = zeros(Iter, 2); %preallocated variables
descriptives = zeros(Iter, 1);
pValue_H = zeros(Iter, 2);

%Loop sampling from population distributions and conducting NHST on vector
for j = 1:Iter %iterations
    
    for i = 1:N*pHo %number of samples from Ho
        distSamples(i,1) = randsample(Ho, 1); %draw samples from Ho    
    end
    
    for i = 1:N*pHa %number of samples from Ha 
        distSamples(i,2) = randsample(Ha, 1); %draw samples from Ha
    end
    
    Participants = [distSamples(:,1); distSamples(:,2)]; %combine samples into a single vector
    Participants = Participants(Participants~=0); %remove zeros (in rare cases)
    
    descriptives(j,1) = mean(Participants); %Save mean of study
%     descriptives(j,2) = std(Participants); %Save std. of study
    
    [~,pValue_H(j,1)] = ttest(Participants(:,1),mean(Ho)); %conduct one-sample t-test on samples drawn and record p-value
%     [~,pValue_H(j,2)] = ttest(Participants(:,1),mean(Ha));

end %for iter

%% Compute ratios

distSkew_Ho = skewness(pValue_H(:,1)); %compute distributional skew of p-values obtained against Ho
% distSkew_Ha = skewness(pValue_H(:,2)); %distributional skew Ha

meta_Pho = zeros(Iter, 3); %preallocated
% meta_Pha = zeros(Iter, 3); %preallocated

%loop to compute p, pp, and p/pp
for i = 1:Iter 
    
    meta_Pho(i,1) = pValue_H(i,1); %p
    meta_Pho(i,2) = (nnz(pValue_H(:,1) <= pValue_H(i,1)) / Iter); %pp
    meta_Pho(i,3) = pValue_H(i,1) / (nnz(pValue_H(:,1) <= pValue_H(i,1))/Iter); %p/pp
    
%     meta_Pha(i,1) = pValue_H(i,2); %p
%     meta_Pha(i,2) = (nnz(pValue_H(:,2) <= pValue_H(i,2)) / Iter); %pp
%     meta_Pha(i,3) = pValue_H(i,2) / (nnz(pValue_H(:,2) <= pValue_H(i,2))/Iter); %p/pp
    
end %for iter

metaEffect_Ho = [.05 .01 .005 .001]'; %columns of p-value cutoffs
% metaEffect_Ha = [.05 .01 .005 .001]';

%matrix: [p-value cutoff (p), number of p-values under cutoff (pp), p/pp]
metaEffect_Ho(1,2) = (nnz(pValue_H(:,1) <= .05)/Iter); %number of sig results p <.05
metaEffect_Ho(2,2) = (nnz(pValue_H(:,1) <= .01)/Iter); %<.01
metaEffect_Ho(3,2) = (nnz(pValue_H(:,1) <= .005)/Iter); %<.005
metaEffect_Ho(4,2) = (nnz(pValue_H(:,1) <= .001)/Iter); %<.001

metaEffect_Ho(1,3) = metaEffect_Ho(1,1) / metaEffect_Ho(1,2); %p/pp .05
metaEffect_Ho(2,3) = metaEffect_Ho(2,1) / metaEffect_Ho(2,2); %p/pp .05
metaEffect_Ho(3,3) = metaEffect_Ho(3,1) / metaEffect_Ho(3,2); %p/pp .005
metaEffect_Ho(4,3) = metaEffect_Ho(4,1) / metaEffect_Ho(4,2); %p/pp .001

metaEffect_Ho(1,4) = 1 - metaEffect_Ho(1,3); %p/pp .05
metaEffect_Ho(2,4) = 1 - metaEffect_Ho(2,3); %p/pp .05
metaEffect_Ho(3,4) = 1 - metaEffect_Ho(3,3); %p/pp .005
metaEffect_Ho(4,4) = 1 - metaEffect_Ho(4,3); %p/pp .001

%Ha
% metaEffect_Ha(1,2) = (nnz(pValue_H(:,2) <= .05)/Iter); %number of sig results Ha <.05
% metaEffect_Ha(2,2) = (nnz(pValue_H(:,2) <= .01)/Iter);  %<.01
% metaEffect_Ha(3,2) = (nnz(pValue_H(:,2) <= .005)/Iter); %<.005
% metaEffect_Ha(4,2) = (nnz(pValue_H(:,2) <= .001)/Iter); %<.001
% 
% metaEffect_Ha(1,3) = metaEffect_Ha(1,1) / metaEffect_Ha(1,2); %number of sig results Ho <.05
% metaEffect_Ha(2,3) = metaEffect_Ha(2,1) / metaEffect_Ha(2,2); %<.01
% metaEffect_Ha(3,3) = metaEffect_Ha(3,1) / metaEffect_Ha(3,2); %<.005
% metaEffect_Ha(4,3) = metaEffect_Ha(4,1) / metaEffect_Ha(4,2); %<.001