% NHST Simulation: full-sample and sequential sample NSHT.

% Author: Patrick R. Heck, 06/2016.
% Written in MATLAB 7.10.0 (R2010a)

% Settings (can change in code): 
% Iter = 1000 %number of means to compute
% p(H) = .5 %proportion of samples to draw from Ho and Ha
% N = 10 %number of samples to draw for each iteration

%Note: all tests against the alternative hypothesis Ha are commented out
%but can be run.

%Note: for full sample testing, remove if/break lines 65-66 and categorize
%only on the full sample of N = 50 (last row in data variable).

%current settings take 5-10 minutes to run. Decrease Iter for faster speed.

clear all

d = 1;

%Create population distributions
Ho = 50 + 10.*randn(100000,1); %mean = 50, SD = 10
Ha = (mean(Ho) + d*std(Ho)) + 10.*randn(100000,1); %+.5 SD above Ho (M = 55, SD = 10)

%   Ha = (mean(Ho) + 10.*randn(100000,1)); %+0 SD above Ho (M= 50, SD = 10) %for sample testing with no effect.


Iter = 1000; %number of iterations: means to compute
N = 10; %number of samples to compute first mean
pHo = .5; %proportion of samples to come from Ho
pHa = (1 - pHo); %proportion of samples to come from Ha

allData = {}; %define cell

for k = 1:Iter %iterations loop through rounds of sequential or full sample
    
    clearvars -except k Ho Ha N pHo pHa allData Iter %keep vars through loop
    
    for i = 1:N*pHo %number of samples from Ho
        distSamples(i,1) = randsample(Ho, 1); %samples from Ho
    end
    
    for i = 1:N*pHa %number of samples from Ha
        distSamples(i,2) = randsample(Ha, 1); %samples from Ha
    end
    
    Participants = [distSamples(:,1); distSamples(:,2)]; %combine samples into a vector
    % Participants = Participants(Participants~=0); %remove zeros (if needed)
    
    sampleMean = mean(Participants);
    
    %Run t-test on sample of 10 against Ho and Ha
    [decision_H,pValue_H,~,stats_H] = ttest(Participants,mean(Ho)); %gives p-value against Ho
    [decision_nH,pValue_nH,~,stats_nH] = ttest(Participants,mean(Ha)); %gives p-value against Ha
          
    %get pdf of t-stat using tpdf function
    densities_H = tpdf(stats_H.tstat,stats_H.df);
    densities_nH = tpdf(stats_nH.tstat,stats_nH.df);
    
    %compute p(H|D)
    x_term = ( (densities_H ./ densities_nH) .* (pHo ./ pHa) ); %see Krueger proof for X
    pHD_dens = 1 ./ ((1./x_term) + 1);
    
    pD = (pHo) * (pValue_H) + (((1 - pHo)) * (pValue_nH)); %unused

    %Compute and store info in a vector
    data(1,1) = pHo; %p(H)
    data(1,2) = pHa; %p(~H)
    data(1,3) = pValue_H; %p(D|H)
    data(1,4) = pValue_nH; %p(D|~H)
    data(1,5) = pD;  %compute p(D)
    data(1,6) = pHD_dens; %compute p(H|D)
    data(1,7) = N; %N
    data(1,8) = sampleMean; %mean
    data(1,9) = std(Participants); %SD
    
    %Add one more participant from each population until significant or N = 50
    for j = 1:20; %increase from 10 per cell to 50 by adding 2, 20 times.
        
        %FOR FULL SAMPLE TESTING, remove if/break lines and categorize
        %only on the full sample of N = 50 (last row in data variable).
        
        if data(j,3) <= .05 %stop if significance achieved %comment out for full testing
            break %comment out for full testing
             
        elseif data(j,3) > .05 %else keep adding and testing %comment out for full testing
            Participants = [Participants; randsample(Ho, 1); randsample(Ha,1)]; %add one more sample from each population
            
            [decision_H,pValue_H,~,stats_H] = ttest(Participants,mean(Ho)); %gives p-value against Ho
            [decision_nH,pValue_nH,~,stats_nH] = ttest(Participants,mean(Ha)); %gives p-value against Ha
            
            %get pdf of t-stat using tpdf function
            densities_H = tpdf(stats_H.tstat,stats_H.df);
            densities_nH = tpdf(stats_nH.tstat,stats_nH.df);
            
            %compute p(H|D)
            x_term = ( (densities_H ./ densities_nH) .* (pHo ./ pHa) ); %see Krueger proof for X
            pHD_dens = 1 ./ ((1./x_term) + 1);
            
            pD = (pHo) * (pValue_H) + (((1 - pHo)) * (pValue_nH)); %unused

            %expand the data vector into a matrix for each sequential test
            %Note that now data is a 9 variables X 21 tests (initial test
            % + 20 more) matrix.
            
            data(j+1,1) = pHo; %p(H)
            data(j+1,2) = pHa; %p(~H)
            data(j+1,3) = pValue_H; %p(D|H)
            data(j+1,4) = pValue_nH; %p(D|~H)
            data(j+1,5) = pD;  %compute p(D)
            data(j+1,6) = pHD_dens; %compute p(H|D)
            data(j+1,7) = length(Participants); %N
            data(j+1,8) = mean(Participants); %Mean
            data(j+1,9) = std(Participants); %SD
         end %comment out for full testing
    end
    
    allData{k,1} = data; %store data matrix in cell for each iteration.
    
end %k

%% Categorize iterations

%Determine whether the stopping rule was met: if data goes to 21 tests, no
%sig was achieved.
for i = 1:Iter 
    simSize(i,:) = (size(allData{i,1}));
end

simLength = simSize(:,1); %trim off unnecessary cell widths

% % for full testing with no effect
% FA_noEffect = zeros(Iter,1);
% for i = 1:Iter
%     if allData{i,1}(end,3) <= .05
%        FA_noEffect(i,1) = 1;
%     end
% end
% proportion_FA_noEffect = sum(FA_noEffect) / Iter;
% 
%categorize each test into a Hit, False Alarm, Miss, or Correct Rejection
%factors are IF p(H|D) is <= .5 (M, H) or > .5 (FA, CR)
%and IF significance was ever achieved in the 21 t-tests

%USE FOR SEQUENTIAL TESTING

for i = 1:Iter
    if simLength(i,1)     == max(simLength) && allData{i,1}(end,6) <= .5;
    errorCat(i,1) = 3; %M: no significance, but p(H|D) < .5
    elseif simLength(i,1) == max(simLength) && allData{i,1}(end,6) > .5;
    errorCat(i,1) = 4; %CR: no significance, and p(H|D) > .5
    elseif simLength(i,1)  < max(simLength) && allData{i,1}(end,6) <= .5;
    errorCat(i,1) = 1; %H: significance, and p(H|D) < .5
    elseif simLength(i,1)  < max(simLength) && allData{i,1}(end,6) > .5;
    errorCat(i,1) = 2; %FA: significance, but p(H|D) > .5
    end
end

%USE FOR FULL SAMPLE TESTING
% for i = 1:Iter
%     if allData{i,1}(end,3) <= .05 && allData{i,1}(end,6) <= .50
%         errorCat(i,1) = 1; %hit
%     elseif allData{i,1}(end,3) <= .05 && allData{i,1}(end,6) > .50
%         errorCat(i,1) = 2; %FA
%     elseif allData{i,1}(end,3) > .05 && allData{i,1}(end,6) <= .50
%         errorCat(i,1) = 3; %M
%     elseif allData{i,1}(end,3) > .05 && allData{i,1}(end,6) > .50
%         errorCat(i,1) = 4; %CR
%     end
% end
    
% Hist count number of categories observed. 1 = H, 2 = FA, 3 = M, 4 = CR.
catFreqs = (histc(errorCat,[1 2 3 4]))./Iter; %count frequencies

catPlot = bar(catFreqs);
set(gca, 'XTickLabel', [1 2 3 4]);
set(gca, 'YLim', [0 1]);
xlabel('Category Proportions', 'FontSize', 14)
ylabel('%', 'FontSize', 14)
title('d = XX, N = YY per condition up to 50', 'FontSize', 14)

%% Stack sims to get correlation

for i = 1:Iter
pDH_mat(i,1) = allData{i,1}(end,3);
pDnH_mat(i,1) = allData{i,1}(end,4);
end

pDH_pDnH_Corr = corr(pDH_mat,pDnH_mat);