% NHST Simulation: Confidence interval widths and variance in means.

% Author: Patrick R. Heck, 06/2016.
% Written in MATLAB 7.10.0 (R2010a)

% Settings (can change in code): 
% Iter = 10000 %number of means to compute
% p(H)=0 %proportion of samples to draw from Ho and Ha (here, all from Ha)
% nSteps 10 = number of steps to vary N
% N = nSteps * 10 %number of samples to draw for each iteration (set to
% 10:10:100)

%Note: all tests against the alternative hypothesis Ha are commented out
%but can be run.

%current settings take ~10 minutes to run. Decrease Iter or nSteps for faster speed.

clear all

%Create population distributions
Ho = 50 + 10.*randn(100000,1); %mean = 50, SD = 10
Ha = (mean(Ho) + .5*std(Ho)) + 10.*randn(100000,1); %+.5 SD above Ho (M = 55, SD = 10)

pValue = {}; %define cells for loop storage
confInt = {};
meanVar = {};

nSteps = 10; % 10 levels of N, from 10 to 100

for s = 1:nSteps; %for 10 levels of N, 10 to 100
    
    Iter = 10000; %number of iterations
    N = s*10; %number of samples to compute mean, looped through nSteps
    pHo = 0; %proportion of samples to come from Ho %NOT NEEDED IN THIS SIM
    pHa = (1 - pHo); %proportion of samples to come from Ha
    
    distSamples = zeros(Iter, 2);
    descriptives = zeros(Iter, 1);
    pValue_H = zeros(Iter, 2);
    
    for j = 1:Iter %iterations
        
        for i = 1:N*pHo %number of samples from Ho %NOT NEEDED IN THIS SIM
            distSamples(i,1) = randsample(Ho, 1); %samples from Ho
        end
        
        for i = 1:N*pHa %number of samples from Ha
            distSamples(i,2) = randsample(Ha, 1); %samples from Ha
        end
        
        Participants = [distSamples(:,1); distSamples(:,2)]; %make participants into a vector
        Participants = Participants(Participants~=0); %remove zeros (rare cases)
        
        descriptives(j,1) = mean(Participants); %Save mean of study
        %     descriptives(j,2) = std(Participants); %Save std. of study
        
        [~,pValue_H(j,1),ciHo] = ttest(Participants(:,1),mean(Ho)); %t-test against Ho
%         [~,pValue_H(j,2),ciHa] = ttest(Participants(:,1),mean(Ha)); %t-test against Ha

        confIntHo(j,1) = ciHo(1,1); %lower bound CI
        confIntHo(j,2) = ciHo(2,1); %upper bound CI
        confIntHo(j,3) = ciHo(2,1) - ciHo(1,1); %difference between bounds

%         confIntHa(j,1) = ciHa(1,1); %lower bound 
%         confIntHa(j,2) = ciHa(2,1); %upper bound
%         confIntHa(j,3) = ciHa(2,1) - ciHa(1,1); %difference
    end
    
    %store results in cell
    confInt{s,1} = confIntHo; %confidence interval bounds and difference
    pValue{s,1} = pValue_H(:,1); %NHST p-value
    meanVar{s,1} = std(descriptives); %variability in computed means
    
end

%% Get output

for q = 1:nSteps
    
    meanCI(q,1) = mean(confInt{q,1}(:,3)); %Mean CI width
    mdnCI(q,1) = median(confInt{q,1}(:,3)); %Median CI width
    meanP(q,1) = mean(pValue{q,1}); %Mean p-value
    mdnP(q,1) = median(pValue{q,1}); %Median p-value
    
end


%% ALL CODE BELOW IS TEST CODE TO OBTAIN REPORTED VALUES. CAN ALSO BE DONE IN EXCEL.

% %% Test means of CI for replication
% 
% for s = 1:nSteps
%     for q = 1:Iter
%         confInt{s,1}(q,4) = mean([confInt{s,1}(q,1) confInt{s,1}(q,2)]);
%     end
% end
% 
% meanContained = {};
% 
% for s = 1:nSteps
%     for i = 1:Iter
%         meanContained{s,1}(i,1) = sum(confInt{s,1}(i,4) > confInt{s,1}(:,1) & confInt{s,1}(i,4) < confInt{s,1}(:,2)) - 1; %subtract one because every mean is in it's own CI.
%     end
% end
% 
% for s = 1:nSteps
%     
%     aveContained(s,1) = mean(meanContained{s,1}); %Frequency
%     aveContained(s,2) = aveContained(s,1) / (Iter-1); %Proportion
%     aveContained(s,3) = median(meanContained{s,1}); %Median
%     aveContained(s,4) = aveContained(s,3) / (Iter-1);
%     aveContained(s,5) = std(meanContained{s,1}); %std.dev
%     aveContained(s,6) = aveContained(s,5) / (Iter-1);
%     
% end
% 
% %% Test p-values for replication
% 
% for s = 1:nSteps %how many sig results total
%     for i = 1:Iter
%         signif1(s,1) = sum(pValue{s,1}(:,1) <= .05);
%         signif1(s,2) = sum(pValue{s,1}(:,1) > .05);
%         signif1(s,3) = (signif1(s,1) - 1) / Iter;
%         signif1(s,4) = (signif1(s,2) - 1) / Iter;
%     end
% end
% 








